/*
 * get_song_data.c - part of rockantenne.c, a plugin for the Video Disk Recorder
 *
 * See the README file for copyright information and how to reach the author.
 *
 */

#include "rockantenne.h"

int str_len(const char* in) {
  register const char* t=in;
  for (;;) {
    if (!*t) break; ++t;
    if (!*t) break; ++t;
    if (!*t) break; ++t;
    if (!*t) break; ++t;
  }
  return (int)(t-in);
}
 
#ifdef CLASSIC

int str_copyn(char *out,const char *in, unsigned long limit) {
  register char* s=out;
  register const char* t=in;
  register const char* u=t+limit;
  for (;;) {
    if (t>=u) { *s='\0'; break; }
    if (!(*s=*t)) break;
    ++s; ++t;
  }
  return (int)(s-out);
}

int str_diffn(const char* a, const char* b, unsigned long limit) {
  register const unsigned char* s=(const unsigned char*)a;
  register const unsigned char* t=(const unsigned char*)b;
  register const unsigned char* u=t+limit;
  register int j;
  j=0;
  for (;;) { if (t>=u) break; if ((j=(*s-*t))) break; if (!*t) break; ++s; ++t; }
  return j;
}

int str_index(const char* a, const char* b) {
  register const char* s=a;
  register int t=str_len(b);
  register const char* u=s+str_len(s)-t;
  for (;;) {
    if (s>u) break;
    if (!str_diffn(s,b,t)) return s-a;
    ++s;
  }
  return -1;
}

int do_http_get(char *buf, int buflen) {
  char get_string[]=STR_HTTP_GET;
  struct sockaddr_in server_sockaddr;
  int sockfd;
  register int bl=buflen;
  register char *b=buf;
  register int i=0;
  register int newlines=0;
  fd_set wfds; 
  struct timeval tv; 

  if (inet_aton( STR_SERVER_IP, &server_sockaddr.sin_addr)!=0) {
    server_sockaddr.sin_family= AF_INET;
    server_sockaddr.sin_port= htons( 80);
    if ((sockfd=socket(PF_INET,SOCK_STREAM,0))!=-1) {
      fcntl(sockfd, F_SETFL,O_RDWR|O_NONBLOCK);
      if (connect(sockfd,(struct sockaddr *)&server_sockaddr,sizeof(server_sockaddr))<0) {
        if (errno != EINPROGRESS) return 0;
      }
      tv.tv_sec = TIMEOUT_SECS;
      tv.tv_usec = 0;
      FD_ZERO(&wfds);
      FD_SET(sockfd, &wfds);
      if (select(sockfd+1, NULL, &wfds, NULL, &tv) <= 0) return 0;
      fcntl(sockfd, F_SETFL,O_RDWR);
      if (write(sockfd,get_string,str_len(get_string))==-1) return 0;
      if ((bl=read(sockfd,b,buflen))>3) {
        if (!close(sockfd)) {
          b[bl]='\0';
          for (;;) {
            if ((newlines>1) || (i>=bl)) break;
            if (b[i]!='\r') {
              if (b[i]=='\n') ++newlines; else newlines=0;
            }
            ++i;
          }
          if (newlines>1) return i;
        }
      }
    }
  }
  return 0; /* no end of http header found */
}

int get_song_data(char *http_buf, int index, struct song_data *song) {
  register int i;
  register char *p;
  char ip_str[]=INTERPRET_STR;
  char tt_str[]=TITLE_STR;
  char nx_str[]=NEXT_STR;

  i = index;
  p = http_buf + i;
  i = str_index(p, ip_str);
  if (i<0) return 10;
  p += i + str_len(ip_str);
  while ((*p==' ') || (*p=='>')) ++p;
  song->interpret = p;
  i = str_index(p, tt_str);
  if (i<0) return 10;
  p[i] = ':';
  p[i+1] = '\0';
  p += i + str_len(tt_str);
  while (*p==' ') ++p;
  song->title = p;
  i = str_index(p, nx_str);
  if (i<0) return 10;
  p[i++] = '\n';
  p[i] = '\0';
  return 0;
}

long get_time_long() {
  register long tl;
  time_t tp;
  struct tm *t;

  tp = time(NULL);
  t = localtime(&tp);
  tl = t->tm_yday * 1000;
  tl += t->tm_hour;
  tl *= 100;
  tl += t->tm_min;
  tl *= 100;
  tl += t->tm_sec;
  return tl;
}

#else

int do_net_get(char *buf, int buflen) {
  struct sockaddr_in server_sockaddr;
  int sockfd;
  register int bl=buflen;
  register char *b=buf;
  fd_set wfds; 
  struct timeval tv; 

  if (inet_aton( STR_SERVER_IP, &server_sockaddr.sin_addr)!=0) {
    server_sockaddr.sin_family= AF_INET;
    server_sockaddr.sin_port= htons( 4488);
    if ((sockfd=socket(PF_INET,SOCK_STREAM,0))!=-1) {
      fcntl(sockfd, F_SETFL,O_RDWR|O_NONBLOCK);
      if (connect(sockfd,(struct sockaddr *)&server_sockaddr,sizeof(server_sockaddr))<0) {
        if (errno != EINPROGRESS) return -1;
      }
      tv.tv_sec = TIMEOUT_SECS;
      tv.tv_usec = 0;
      FD_ZERO(&wfds);
      FD_SET(sockfd, &wfds);
      if (select(sockfd+1, NULL, &wfds, NULL, &tv) <= 0) return -1;
      fcntl(sockfd, F_SETFL,O_RDWR);
      bl=read(sockfd,b,buflen);
      b[bl]='\0';
      close(sockfd);
      return bl;
    }
  }
  return -1;
}

#endif
