/*
 * rockantenne.c: A plugin for the Video Disk Recorder
 *
 * See the README file for copyright information and how to reach the author.
 *
 * $Id$
 */

/*
 * CLASSIC mode is a http-get directly from Radio Rockantenne server
 * the standard mode uses an alternate server which provides *4* titles
 *
#define CLASSIC
 */

#include "rockantenne.h"
#include <vdr/plugin.h>
#include <vdr/skins.h>
#include <vdr/interface.h>
#include <vdr/osd.h>

#define CACHETIME 10
#define TIMEOUT_SECS 4
/* very ugly, but very simple X-) */
#include "get_song_data.c"

static const char *VERSION        = "0.1.0";
static const char *DESCRIPTION    = "Show song(s) played on Rockantenne";
static const char *MAINMENUENTRY  = "Rockantenne";

#ifndef CLASSIC

class cRockantenneDisplay : public cOsdObject {
private:
  cOsd *osd;
  const cFont* pFont;
  int lineHeight;
public:
  cRockantenneDisplay(void);
  ~cRockantenneDisplay();
  virtual void Show(void);
  virtual eOSState ProcessKey(eKeys Key);
};

//cRockantenneDisplay::cRockantenneDisplay(void) : cOsdObject(true) {
cRockantenneDisplay::cRockantenneDisplay(void) {
  osd = NULL;
  pFont = NULL;
}

cRockantenneDisplay::~cRockantenneDisplay() {
  delete osd;
}

void cRockantenneDisplay::Show(void) {
  register char *p;
  register int len;
  register int c;
  char http_buf[BUFLEN+4];
  char *titles[4];

  if (!pFont) {
    pFont = cFont::GetFont(fontSml);
    pFont->SetCode(code_iso8859_1);
    lineHeight = pFont->Height();
  }
  const int screenWidth = Setup.OSDWidth;
  const int screenHeight = Setup.OSDHeight;
  const int width = (((int) (screenWidth / 13)) * 12 ) - 1 ;
  osd = cOsdProvider::NewOsd(Setup.OSDLeft+((screenWidth - width) / 2),
                             Setup.OSDTop+screenHeight-(5*lineHeight)+12);
  if (osd) {
    tArea Areas[] = { { 0, 0, width, (4*lineHeight)-1, 2 } };
    if (osd->SetAreas(Areas, 1) != oeOk) {
      return;
    }
    p = http_buf;
    len = do_net_get(p, BUFLEN);
    if (len<=0) return;
    p[len] = '\n'; // just for security
    p[len+1] = '\n'; // just for security
    p[len+2] = '\n'; // just for security
    p[len+3] = '\n'; // just for security
    for(c=0;c<4;c++) {
      titles[c] = p;
      while(*p!='\n') ++p;
      *p = '\0';
      ++p;
    }
    osd->DrawRectangle(0, 0, width, 4*lineHeight, clrGray50);
    osd->DrawText(0,0,titles[0], clrYellow, clrTransparent, pFont);
    osd->DrawText(0,lineHeight,titles[1], clrYellow, clrTransparent, pFont);
    osd->DrawText(0,lineHeight*2,titles[2], clrYellow, clrTransparent, pFont);
    osd->DrawText(0,lineHeight*3,titles[3], clrYellow, clrTransparent, pFont);
    osd->Flush();
  }
}

eOSState cRockantenneDisplay::ProcessKey(eKeys Key) {
  eOSState state = cOsdObject::ProcessKey(Key);
  if (state == osUnknown) {
    switch (Key & ~k_Repeat) {
      case kOk: case kBack: return osEnd;
      default: return state;
    }
  }
  return state;
}

#endif

class cPluginRockantenne : public cPlugin {
private:
  // Add any member variables or functions you may need here.
  struct song_data stsong;
  char http_buf[BUFLEN+1];
  char song[STRING_LEN+1];
  long songtime;
public:
  cPluginRockantenne(void);
  virtual ~cPluginRockantenne();
  virtual const char *Version(void) { return VERSION; }
  virtual const char *Description(void) { return DESCRIPTION; }
  virtual const char *CommandLineHelp(void);
  virtual bool ProcessArgs(int argc, char *argv[]);
  virtual bool Initialize(void);
  virtual bool Start(void);
  virtual void Stop(void);
  virtual void Housekeeping(void);
  virtual void MainThreadHook(void);
  virtual cString Active(void);
  virtual const char *MainMenuEntry(void) { return MAINMENUENTRY; }
  virtual cOsdObject *MainMenuAction(void);
  virtual cMenuSetupPage *SetupMenu(void);
  virtual bool SetupParse(const char *Name, const char *Value);
  virtual bool Service(const char *Id, void *Data = NULL);
  virtual const char **SVDRPHelpPages(void);
  virtual cString SVDRPCommand(const char *Command, const char *Option, int &ReplyCode);
  };

cPluginRockantenne::cPluginRockantenne(void) {
  // Initialize any member variables here.
  // DON'T DO ANYTHING ELSE THAT MAY HAVE SIDE EFFECTS, REQUIRE GLOBAL
  // VDR OBJECTS TO EXIST OR PRODUCE ANY OUTPUT!
  songtime = 0;
}

cPluginRockantenne::~cPluginRockantenne() {
  // Clean up after yourself!
}

const char *cPluginRockantenne::CommandLineHelp(void) {
  // Return a string that describes all known command line options.
  return NULL;
}

bool cPluginRockantenne::ProcessArgs(int argc, char *argv[]) {
  // Implement command line argument processing here if applicable.
  return true;
}

bool cPluginRockantenne::Initialize(void) {
  // Initialize any background activities the plugin shall perform.
  return true;
}

bool cPluginRockantenne::Start(void) {
  // Start any background activities the plugin shall perform.
  return true;
}

void cPluginRockantenne::Stop(void) {
  // Stop any background activities the plugin shall perform.
}

void cPluginRockantenne::Housekeeping(void) {
  // Perform any cleanup or other regular tasks.
}

void cPluginRockantenne::MainThreadHook(void) {
  // Perform actions in the context of the main program thread.
  // WARNING: Use with great care - see PLUGINS.html!
}

cString cPluginRockantenne::Active(void) {
  // Return a message string if shutdown should be postponed
  return NULL;
}

cOsdObject *cPluginRockantenne::MainMenuAction(void) {

#ifdef CLASSIC
  int i;
  register char *p;
  long ttime;

  // Perform the action when selected from the main VDR menu.
  p = song;
  ttime = get_time_long();
  if (ttime > (songtime+CACHETIME)) {
    i = do_http_get(http_buf, BUFLEN);
    if (i) {
      i = get_song_data(http_buf, i, &stsong);
      if (i!=0) {
        Skins.Message(mtStatus, "<error while parsing data>");
      } else {
        i = str_copyn(p, stsong.interpret, STRING_LEN/2);
        p += i;
        i = str_copyn(p, stsong.title, STRING_LEN/2);
        p[i] = '\0';
        p = song;
        // limit string
        p[42] = '\0';
        songtime = ttime;
        Skins.Message(mtStatus, p);
      }
    } else {
      Skins.Message(mtStatus, "<error on http-get>");
    }
  } else {
    Skins.Message(mtStatus, p);
  }
  Interface->Wait(Setup.OSDMessageTime);
  Skins.Message(mtStatus, NULL);
  return NULL;
#else
  return new cRockantenneDisplay;
#endif
}

cMenuSetupPage *cPluginRockantenne::SetupMenu(void) {
  // Return a setup menu in case the plugin supports one.
  return NULL;
}

bool cPluginRockantenne::SetupParse(const char *Name, const char *Value) {
  // Parse your own setup parameters and store their values.
  return false;
}

bool cPluginRockantenne::Service(const char *Id, void *Data) {
  // Handle custom service requests from other plugins
  return false;
}

const char **cPluginRockantenne::SVDRPHelpPages(void) {
  // Return help text for SVDRP commands this plugin implements
  return NULL;
}

cString cPluginRockantenne::SVDRPCommand(const char *Command, const char *Option, int &ReplyCode) {
  // Process SVDRP commands this plugin implements
  return NULL;
}

VDRPLUGINCREATOR(cPluginRockantenne); // Don't touch this!
